
set ALKS_TclScripts [list \
    "ALKSPerformance.tcl"
]

foreach elm $ALKS_TclScripts {
    source [file join [file dirname [info script]] $elm]
}

### Namespace ALKS ###
namespace eval ::ALKS {

# Default values used for SignalMapping
array set SignalMappingNValues [list \
    ALKS_LongCtrl_Time2Collision AccelCtrl.ACC.Time2Collision \
    ALKS_LongCtrl_DesiredDist AccelCtrl.ACC.DesiredDist\
    ALKS_LongCtrl_MinDist [IFileRead Data/Vehicle/DemoCar_UNECE_ADAS AccelCtrl.ACC.DistMin] \
    ALKS_LongCtrl_MinSpd 0 \
    ALKS_LongCtrl_MaxSpd 60 \
    ALKS_LongCtrl_DesiredSpd AccelCtrl.ACC.DesiredSpd \
    ALKS_LongCtrl_IsActive AccelCtrl.ACC.IsActive \
    ALKS_AEB_SwitchedOn LongCtrl.AEB.SwitchedOn  \
    ALKS_LatCtrl_SwitchedOn LatCtrl.LKAS.SwitchedOn \
    ALKS_AEB_IsActive LongCtrl.AEB.IsActive \
    ALKS_LatCtrl_IsActive LatCtrl.AEB.IsActive \
    ALKS_LongCtrl_Sensor_Forward_relvTgt_dtct Sensor.Object.FrontRadar.relvTgt.dtct \
    ALKS_LongCtrl_Sensor_ForwardDist_x Sensor.Object.FrontRadar.relvTgt.NearPnt.ds.x\
    ALKS_LongCtrl_Sensor_ForwardDist_y Sensor.Object.FrontRadar.relvTgt.NearPnt.ds.y \
    ALKS_LongCtrl_Sensor_ForwardDist_p Sensor.Object.FrontRadar.relvTgt.NearPnt.ds_p \
    ALKS_LongCtrl_Sensor_Backward_relvTgt_dtct Sensor.Object.RearRadar.relvTgt.dtct \
    ALKS_LongCtrl_Sensor_DistBackward_x  Sensor.Object.RearRadar.relvTgt.NearPnt.ds.x \
    ALKS_LongCtrl_Sensor_DistBackward_y Sensor.Object.RearRadar.relvTgt.NearPnt.ds.y \
    ALKS_LongCtrl_Sensor_DistBackward_p  Sensor.Object.RearRadar.relvTgt.NearPnt.ds_p \
    ALKS_LatCtrl_Sensor_relvTgt_dtct Sensor.Object.BSDLeft.relvTgt.dtct \
    ALKS_LatCtrl_Sensor_DistLeft_x Sensor.Object.BSDLeft.relvTgt.NearPnt.ds.x \
    ALKS_LatCtrl_Sensor_DistLeft_y Sensor.Object.BSDLeft.relvTgt.NearPnt.ds.y \
    ALKS_LatCtrl_Sensor_DistLeft_p Sensor.Object.BSDLeft.relvTgt.NearPnt.ds_p\
    ALKS_LatCtrl_Sensor_relvTgtRight_dtct Sensor.Object.BSDRight.relvTgt.dtct \
    ALKS_LatCtrl_Sensor_DistRight_x Sensor.Object.BSDRight.relvTgt.NearPnt.ds.x\
    ALKS_LatCtrl_Sensor_DistRight_y Sensor.Object.BSDRight.relvTgt.NearPnt.ds.y \
    ALKS_LatCtrl_Sensor_DistRight_p Sensor.Object.BSDRight.relvTgt.NearPnt.ds_p	\
    ]

### SetStartCondition ###\
#
# Calculate vut start position according to its velocity
#
proc SetStartCondition {v_vut_kph name} {

    if {${v_vut_kph} eq ""} {return}
    set v_vut_m/s [expr {${v_vut_kph}/3.6}]
		
    # Distance Traffic Object - Distance Acceleration Phase with 1 m/s - Distance const velocity (- const Distance for lower vru := 10m)
    set s_offset 15
    set a_m/s2 1.0
    
    if {[string match "*LaneKeeping_leading_GVT" $name] || [string match "*LaneKeeping_swerving_GVT*" $name] || [string match "*LaneKeeping_GVT driving close beside*" $name]} {
	
	set s_TrfObject [expr {10100 - [NamedValue get v_GVT]/3.6 * 600}]
	set t_const 15.0
	NamedValue set sTHW [expr {[NamedValue get THW] * ([NamedValue get v_vut]/3.6)}]
    } 	elseif {[string match "*Stationary_target" $name]} {
	
	set s_TrfObject  400
	set t_const 25
	if {[NamedValue get curved] ==1 } {
		set s_TrfObject  650
	}
    } 	elseif {[string match "*Crossing_pedestrian_left" $name] || [string match "*Crossing_pedestrian_right" $name] } {
	
	set s_TrfObject 250
	set t_const 20.0
	set s_TrfObject  [expr {  24*([NamedValue get v_vut]/3.6)  }]
	NamedValue set StartPos_s  [expr {  24*([NamedValue get v_vut]/3.6) +($s_TrfObject - ((${v_vut_m/s}*${v_vut_m/s} )/( 2*${a_m/s2})) - (${v_vut_m/s}*10) - $s_offset )}]
    
    } elseif {[string match "*Crossing_pedestrian_curved" $name] } {
	    
	set s_TrfObject 1335.4
	set maxValue 100
	set t_const 20.0
    
    } elseif {[string match "*Multiple_obstacles_blocking_the_lane" $name]} {
	
	set s_TrfObject 250
	set maxValue 100
	set t_const 20.0
	
	if {[NamedValue get curved] ==1 } {
	    set s_TrfObject  650
	}
	set s_TrfObject2 [expr {$s_TrfObject+10 }]
	NamedValue set s_GVT2 $s_TrfObject2
	NamedValue set	 t_GVT2 0
	
	    
	if {[NamedValue get v_vut] <= 10 } { 
	set t_const 40
	}

    } elseif {[string match "*Following_constant_speed_deceleration" $name] || [string match "*Following_varying_speed_deceleration" $name]} {
	
	set s_TrfObject [expr {6000 - [NamedValue get v_GVT]/3.6 * 300}]
	set t_const 10.0
	NamedValue set sTHW [expr {[NamedValue get THW] * ([NamedValue get v_vut]/3.6)}]
	if {[NamedValue get curved] ==1 } {
	    
	    set s_TrfObject [expr {10000 - [NamedValue get v_GVT]/3.6 * 300}]

	}

    } elseif {[string match "*CutIn" $name]} {
    
	NamedValue set sTTC [expr {[NamedValue get TTC] * ([NamedValue get v_vut]/3.6 - [NamedValue get v_GVT]/3.6)}]
	NamedValue set t_Change_CutIn [expr {3.5/ [NamedValue get Vy]}]
	
	NamedValue set sTTCOffset [expr {0.8/ [NamedValue get Vy] * ([NamedValue get v_vut]/3.6 - [NamedValue get v_GVT]/3.6)}]
	set s_TrfObject  [expr {( [NamedValue get v_vut] - [NamedValue get v_GVT] ) /3.6 * 20}] 

	set t_const 25
    
    } else {
	set s_TrfObject 500
	set maxValue 300
	set t_const 15.0
    }	 

    
    set s_start [expr {($s_TrfObject - ((${v_vut_m/s}*${v_vut_m/s} )/( 2*${a_m/s2})) - (${v_vut_m/s}*$t_const) - $s_offset )}]

    ### Definition of boundary for s_vut_offset
    # min. Vaule:= 0
    # max. Vaule:= Depending on TestRun	
    set s_start [expr max(0,$s_start)]
   # set s_start [expr min($maxValue,$s_start)]
    
    NamedValue set s_vut_offset $s_start
    NamedValue set s_GVT $s_TrfObject 

    
    if {[string match "*LaneKeeping_GVT driving close beside*" $name]} {
	    
    NamedValue set s_GVT $s_start

    }

    if {[string match "*CutOut" $name]} {
	
	NamedValue set s_vut_offset  0
	NamedValue set t_Change_CutOut [expr {3.5 / [NamedValue get Vy]}]
	NamedValue set sTHW [expr {[NamedValue get THW] * ([NamedValue get v_vut]/3.6)}]
	NamedValue set StartPos2 [expr { [NamedValue get sTHW] +(60 * [NamedValue get v_vut]/10) }]
	NamedValue set s_GVT [expr { [NamedValue get sTHW] +(150 *  [NamedValue get v_vut]/10 )}]
    
    }
    
    if {[string match "*FoV_lateral" $name]} {
    
	set s_TrfObject  0 
	set s_start 150
	NamedValue set s_vut_offset $s_start
	NamedValue set s_GVT $s_TrfObject 

    }

    return
}

### SignalMapping ###
#
# Mapping between user defined signals and NamedValue used in TestRun info files
#
###
proc SignalMapping {args} {
    variable SignalMappingNValues
    set path2SignalMapping [IFileRead SimParameters "SignalMapping_UNECE_ADAS" "./Data/Config/SignalMapping_UNECE_ADAS"]
    
    if {![file exists $path2SignalMapping]} {
	
	set Msg [l "File '$path2SignalMapping' not found."  "Datei '$path2SignalMapping' nicht gefunden." ]
	PopupMsg err [l "Cannot load Signal Mapping file:\n$Msg"  "Kann Signal Mapping Datei nicht laden:\n$Msg" ]
	return 1
    }
    
    ifile new handle
    handle read $path2SignalMapping errorvar
    
    if {$errorvar ne ""} {
	
	PopupMsg err [l "Cannot load Signal Mapping file:\n$errorvar"  "Kann Signal Mapping Datei nicht laden:\n$errorvar" ]
	ifile delete handle
	return 1
    }
    
    handle getstr "FileIdent" line ""
    if {[scan $line "CarMaker-SignalMapping %d" FileVersion] != 1} {
	
	set Msg [l "invalid/unsupported file format."  "ungltiges/unbekanntes Dateiformat." ]
	PopupMsg err [l "Cannot load Signal Mapping file:\n$Msg"  "Kann Signal Mapping Datei nicht laden:\n$Msg" ]
	ifile delete handle
	return 1
    }
    
    handle listkeys "" all_keys key_list
    
    for {set i 0} {$i < [llength $key_list]} {incr i} {
	
	set key [lindex $key_list $i]
	if {$key eq "FileIdent"} { continue }
	
	if {[info exists SignalMappingNValues($key)]} {
	
		handle get $key var $SignalMappingNValues($key)
		# Code modified
		NamedValue set $key $SignalMappingNValues($key)
	} else {
	
		handle get $key var
		# Code modified
		NamedValue set $key $var
	}
	    #NamedValue set $key $var
    }
    ifile delete handle
    
    return 0
}

### GetTrafficWidth ###
#
# Determine width of traffic object
#
###
proc GetTrafficWidth {nameTR} {
    # read traffic width from TestRun data
    set path2TestRun "Data/TestRun/$nameTR"
    set trf_dim [IFileRead $path2TestRun Traffic.0.Basics.Dimension]

    set TrafficWidth [lindex $trf_dim 1]
    return $TrafficWidth
}

### GetVehicleWidth ###
#
# Calculate width of vehicle under test
#
###
proc GetVehicleWidth {nameTR} {
    # read vehicle width from vehicle data
    # if Vehicle variation is set in TestManager
    if {[KeyValue exists TestRun:Vehicle]} {
	set path2Vehicle Data/Vehicle/[KeyValue get TestRun:Vehicle]

    } else {
	    # use Vehicle linked with TestRun
	    set path2TestRun "Data/TestRun/$nameTR"
	    set vehicle [IFileRead $path2TestRun Vehicle]
	    set path2Vehicle Data/Vehicle/$vehicle
    }

    set OuterSkinRLLP [lindex [IFileRead $path2Vehicle Vehicle.OuterSkin] 1]
    set OuterSKinFURP [expr abs([lindex [IFileRead $path2Vehicle Vehicle.OuterSkin] 4])]

    set WRLPosY [lindex [IFileRead $path2Vehicle Wheel.rl.pos] 1];
    set TireWidthRL [IFileRead Tire2 NomWidth]
    
    set WFLPosY [lindex [IFileRead $path2Vehicle Wheel.fl.pos] 1];
    set TireWidthFL [IFileRead Tire0 NomWidth]
    
    set WidthRL [expr $WRLPosY+($TireWidthRL/2)]
    set WidthFL [expr $WFLPosY+($TireWidthFL/2)]
    
    if {$WidthFL > $WidthRL} {
	set Width $WidthFL
    } else {
	set Width $WidthRL
    }

    if {$TireWidthFL > $TireWidthRL} {
	set TireWidth $TireWidthFL
    } else {
	set TireWidth $TireWidthRL
    }
    
    set WFLRadius [IFileRead Tire0 KinRollRadius]
    set WFLDiagonal [expr (sqrt($WFLRadius*$WFLRadius+($TireWidth/2)*($TireWidth/2)))]
    
    NamedValue set Angle [expr asin($TireWidth/2/$WFLDiagonal)]
    
    if {$OuterSkinRLLP > 0 && ($OuterSkinRLLP >= $Width || $OuterSKinFURP >= $Width)} {
	if {abs($OuterSkinRLLP) >= abs($OuterSKinFURP)} {
	    set VehicleWidth [expr 2.0*$OuterSkinRLLP]
	    } else {
	    set VehicleWidth [expr 2.0*($OuterSKinFURP)]
	    }
    } else {
#       offset between defined wheel position and outer vehicle position for width calculation
#       value is only estimated
	set WhlOffset 0.025;
	set VehicleWidth [expr 2.0*($Width+$WhlOffset)]
    }
    
    NamedValue set TireWidthFL $TireWidthFL
    NamedValue set WFLDiagonal $WFLDiagonal
    NamedValue set VehicleWidth $VehicleWidth

    return
}

### GetVehicleLength ###
#
# Determine length of vehicle under test & reference point x-coordinate for sRoad - coordinate (Fr1)
#
###
proc GetVehicleLength {nameTR} {
    if {[KeyValue exists TestRun:Vehicle]} {
	    set path2Vehicle Data/Vehicle/[KeyValue get TestRun:Vehicle]
    } else {
	    # use Vehicle linked with TestRun
	    set path2TestRun "Data/TestRun/$nameTR"
	    set vehicle [IFileRead $path2TestRun Vehicle]
	    set path2Vehicle Data/Vehicle/$vehicle
    }
    
    set VehicleLength [lindex [IFileRead $path2Vehicle Vehicle.OuterSkin] 3]
    set WFLPosX [lindex [IFileRead $path2Vehicle Wheel.fl.pos] 0]
    set WFRPosX [lindex [IFileRead $path2Vehicle Wheel.fr.pos] 0]
    set WFLPosY [lindex [IFileRead $path2Vehicle Wheel.fl.pos] 1]
    set WFRPosY [lindex [IFileRead $path2Vehicle Wheel.fr.pos] 1]
    set WFLPosZ [lindex [IFileRead $path2Vehicle Wheel.fl.pos] 2]
    set WFRPosZ [lindex [IFileRead $path2Vehicle Wheel.fr.pos] 2]

    set VehicleRefPointsRoad [expr {($WFLPosX + $WFRPosX)/2}]
    set VehicleRefPointsRoadY [expr {($WFLPosY + $WFRPosY)/2}]
    set VehicleRefPointsRoadZ [expr {($WFLPosZ + $WFRPosZ)/2}]

    NamedValue set VehicleLength $VehicleLength
    NamedValue set VehicleRefPointsRoad $VehicleRefPointsRoad
    NamedValue set VehicleRefPointsRoadY $VehicleRefPointsRoadY
    NamedValue set VehicleRefPointsRoadZ $VehicleRefPointsRoadZ
    
    return
}

### GetWRLPos ###
#
# Determine the x-Coordinate of the Rear Left Wheel centre
#
###
proc GetWRLPos {nameTR} {
    if {[KeyValue exists TestRun:Vehicle]} {
	    set path2Vehicle Data/Vehicle/[KeyValue get TestRun:Vehicle]
    } else {
	# use Vehicle linked with TestRun
	set path2TestRun "Data/TestRun/$nameTR"
	set vehicle [IFileRead $path2TestRun Vehicle]
	set path2Vehicle Data/Vehicle/$vehicle
    }
    
    set WRLPosX [lindex [IFileRead $path2Vehicle Wheel.rl.pos] 0];
    set WRRPosX [lindex [IFileRead $path2Vehicle Wheel.rr.pos] 0];
    
    NamedValue set WRLPosX $WRLPosX
}

### CompareResultDate ###
#
# Delete test results & all TS-variables if the date of test results
# differs more than one day from current date
#
###
proc CompareResultDate {args} {
    set difference_days 1
    set ::TS::Date "[clock format [clock seconds] -format %m/%d/%Y]"
    
    if {[info exists ::TS::ResultDate]} {
	set day1 [clock format [clock scan $::TS::ResultDate] -format %j]
	set day2 [clock format [clock scan $::TS::Date] -format %j]

	if {[expr {abs([scan $day2 %d] - [scan $day1 %d])}] > $difference_days} { 
		::TestMgr::ClearTS
		::TestMgr clearresults
		}
	}
}


### SetResultDate ###
#
# Store test result date in TS variables which is used in CompareResultDate for comparision
#
###
proc SetResultDate {args} {
	set ::TS::ResultDate "[clock format [clock seconds] -format %m/%d/%Y]"
}


### GetSystem ###
#
# Delete saved test scores (& optional: test results + all TS-variables),
# if the current system under test differs from previous setting
#
###
proc GetSystem {system args} {
    switch $system {
	ALKS {
		if {[info exists ::TS::System(ALKS)]} {
			if {[NamedValue get EquippedALKS] == 1} {
				set system ALKS
			}
		} else {
			if {[NamedValue get EquippedALKS] == 1} {
				set ::TS::System(ALKS) ALKS
			} else {
				set ::TS::System(ALKS) none
			}
		}

		set ::TS::ALKS [NamedValue get EquippedALKS]
		NamedValue set ALKS_On [set ::TS::ALKS]
	}
	default {}
    }
    
}

### SetSystem ###
#
# Store system information in TS variables which is used in GetSystem for comparision
#
###
proc SetSystem {system args} {
    switch $system {
	ALKS {
		if {![info exists ::TS::ALKS]} {
		    unset -nocomplain ::TS::System(ALKS)
		    return
		}
	
		if {$::TS::ALKS == 1} {
		    set ::TS::System(ALKS) ALKS
		} else {
		    set ::TS::System(ALKS) none
		}
	}
	default {}
    }
}

### SelectTSNamespace ###
#
# Selet used tcl namespace for test score/ result storage
#
###

proc SelectTSNamespace {args} {
global HIL
    
    if {$HIL(HPC.Active)} {
	    set ns TS_tmp
    } else {
	    set ns TS
    }
    
    return $ns
}

### End Namespace ALKS###
}