# set EuroNCAP_TclScripts [list \
	# "AEB_C2C.tcl" \
	# "AEB_VRU.tcl" \
	# "LaneSupportSystems.tcl" \
	# "OverallRating.tcl" \
	# "SpeedAssistSystems.tcl" \
	# # "CM2Spreadsheet.tcl"
# ]

set ELKS_TclScripts [list \
	"ELKSPerformance.tcl" \
]

foreach elm $ELKS_TclScripts {
	source [file join [file dirname [info script]] $elm]
}

### Namespace ELKS ###
namespace eval ::ELKS {

# Default values used for SignalMapping
array set SignalMappingNValues [list \
    ALKS_LongCtrl_Time2Collision AccelCtrl.ACC.Time2Collision \
    ALKS_LongCtrl_DesiredDist AccelCtrl.ACC.DesiredDist\
    ALKS_LongCtrl_MinDist [IFileRead Data/Vehicle/DemoCar_UNECE_ADAS AccelCtrl.ACC.DistMin] \
    ALKS_LongCtrl_MinSpd 0 \
    ALKS_LongCtrl_MaxSpd 60 \
    ALKS_LongCtrl_DesiredSpd AccelCtrl.ACC.DesiredSpd \
    ALKS_LongCtrl_IsActive AccelCtrl.ACC.IsActive \
    ALKS_AEB_SwitchedOn LongCtrl.AEB.SwitchedOn  \
    ALKS_LatCtrl_SwitchedOn LatCtrl.LKAS.SwitchedOn \
    ALKS_AEB_IsActive LongCtrl.AEB.IsActive \
    ALKS_LatCtrl_IsActive LatCtrl.AEB.IsActive \
    ALKS_LongCtrl_Sensor_Forward_relvTgt_dtct Sensor.Object.FrontRadar.relvTgt.dtct \
    ALKS_LongCtrl_Sensor_ForwardDist_x Sensor.Object.FrontRadar.relvTgt.NearPnt.ds.x\
    ALKS_LongCtrl_Sensor_ForwardDist_y Sensor.Object.FrontRadar.relvTgt.NearPnt.ds.y \
    ALKS_LongCtrl_Sensor_ForwardDist_p Sensor.Object.FrontRadar.relvTgt.NearPnt.ds_p \
    ALKS_LongCtrl_Sensor_Backward_relvTgt_dtct Sensor.Object.RearRadar.relvTgt.dtct \
    ALKS_LongCtrl_Sensor_DistBackward_x  Sensor.Object.RearRadar.relvTgt.NearPnt.ds.x \
    ALKS_LongCtrl_Sensor_DistBackward_y Sensor.Object.RearRadar.relvTgt.NearPnt.ds.y \
    ALKS_LongCtrl_Sensor_DistBackward_p  Sensor.Object.RearRadar.relvTgt.NearPnt.ds_p \
    ALKS_LatCtrl_Sensor_relvTgt_dtct Sensor.Object.BSDLeft.relvTgt.dtct \
    ALKS_LatCtrl_Sensor_DistLeft_x Sensor.Object.BSDLeft.relvTgt.NearPnt.ds.x \
    ALKS_LatCtrl_Sensor_DistLeft_y Sensor.Object.BSDLeft.relvTgt.NearPnt.ds.y \
    ALKS_LatCtrl_Sensor_DistLeft_p Sensor.Object.BSDLeft.relvTgt.NearPnt.ds_p\
    ALKS_LatCtrl_Sensor_relvTgtRight_dtct Sensor.Object.BSDRight.relvTgt.dtct \
    ALKS_LatCtrl_Sensor_DistRight_x Sensor.Object.BSDRight.relvTgt.NearPnt.ds.x\
    ALKS_LatCtrl_Sensor_DistRight_y Sensor.Object.BSDRight.relvTgt.NearPnt.ds.y \
    ALKS_LatCtrl_Sensor_DistRight_p Sensor.Object.BSDRight.relvTgt.NearPnt.ds_p	\
	ELKS_LDW_SwitchedOn LatCtrl.LDW.SwitchedOn \
    ELKS_LDW_IsActive LatCtrl.LDW.IsActive \
	ELKS_CDCF_SwitchedOn LatCtrl.LKAS.SwitchedOn \
	ELKS_CDCF_IsActive LatCtrl.LKAS.IsActive \
	ELKS_CDCF_WarnSignal_Visual LatCtrl.LDW.IsActive \
	ELKS_CDCF_WarnSignal_Acousic LatCtrl.LDW.IsActive \
	ELKS_CDCF_Assist_Steer_Trq  LatCtrl.LKAS.AssistTrq \
    ]
	

	


# ### SetStartCondition ###
# #
# # Calculate vut start position according to its velocity
# #
# proc SetStartCondition {v_vut_kph} {

	# if {${v_vut_kph} eq ""} {return}
	# set v_vut_m/s [expr {${v_vut_kph}/3.6}]
	
	# # Distance Traffic Object - Distance Acceleration Phase with 1 m/s - Distance const velocity (- const Distance for lower vru := 10m)
	# set s_offset 15
	# set s_TrfObject 500
	# set a_m/s2 1.0
	# set t_const 15.0
	# set s_start [expr {($s_TrfObject - ((${v_vut_m/s}*${v_vut_m/s} )/( 2*${a_m/s2})) - (${v_vut_m/s}*$t_const) - $s_offset )}]

	# ### Definition of boundary for s_vut_offset
	# # min. Vaule:= 0
	# # max. Vaule:= 300
	# set s_start [expr max(0,$s_start)]
	# set s_start [expr min(300,$s_start)]
	
	# NamedValue set s_vut_offset $s_start
	
	# return
# }

### SignalMapping ###
#
# Mapping between user defined signals and NamedValue used in TestRun info files
#
###
proc SignalMapping {args} {
variable SignalMappingNValues
    set path2SignalMapping [IFileRead SimParameters "SignalMapping_UNECE_ADAS" "./Data/Config/SignalMapping_UNECE_ADAS"]
	
	if {![file exists $path2SignalMapping]} {
		set Msg [l "File '$path2SignalMapping' not found."  "Datei '$path2SignalMapping' nicht gefunden." ]
		PopupMsg err [l "Cannot load Signal Mapping file:\n$Msg"  "Kann Signal Mapping Datei nicht laden:\n$Msg" ]
		return 1
	}
	
	ifile new handle
	handle read $path2SignalMapping errorvar
	
	if {$errorvar ne ""} {
		PopupMsg err [l "Cannot load Signal Mapping file:\n$errorvar"  "Kann Signal Mapping Datei nicht laden:\n$errorvar" ]
		ifile delete handle
		return 1
	}
	
	handle getstr "FileIdent" line ""
	if {[scan $line "CarMaker-SignalMapping %d" FileVersion] != 1} {
		set Msg [l "invalid/unsupported file format."  "ungltiges/unbekanntes Dateiformat." ]
		PopupMsg err [l "Cannot load Signal Mapping file:\n$Msg"  "Kann Signal Mapping Datei nicht laden:\n$Msg" ]
		ifile delete handle
		return 1
	}
	
	handle listkeys "" all_keys key_list
	
	for {set i 0} {$i < [llength $key_list]} {incr i} {
		set key [lindex $key_list $i]
		if {$key eq "FileIdent"} { continue }
		
		if {[info exists SignalMappingNValues($key)]} {
			handle get $key var $SignalMappingNValues($key)
		} else {
			handle get $key var 
		}
		NamedValue set $key $var
	}
	ifile delete handle
	
	return 0
}

# ### GetTrafficWidth ###
# #
# # Determine width of traffic object
# #
# ###
# proc GetTrafficWidth {nameTR} {
	# # read traffic width from TestRun data
	# set path2TestRun "Data/TestRun/$nameTR"
	# set trf_dim [IFileRead $path2TestRun Traffic.0.Basics.Dimension]

	# set TrafficWidth [lindex $trf_dim 1]
	# return $TrafficWidth
# }

### GetVehicleWidth ###
#
# Calculate width of vehicle under test
#
###
proc GetVehicleWidth {args} {
	# read vehicle width from vehicle data
	# if Vehicle variation is set in TestManager
	if {[KeyValue exists TestRun:Vehicle]} {
#       Log "Vehicle: [KeyValue get TestRun::Vehicle]"
		set path2Vehicle Data/Vehicle/[KeyValue get TestRun:Vehicle]

	} else {
		# use Vehicle linked with TestRun
		set path2TestRun Data/TestRun/TestWare/EuroNCAP_2020/Runs/LSS/LSS_LKA_Lines
#         Log screen "TestRun $path2TestRun"
		set vehicle [IFileRead $path2TestRun Vehicle]
#         Log screen "Vehicle: $vehicle"
		set path2Vehicle Data/Vehicle/$vehicle
	}
#     Log screen "path2Vehicle $path2Vehicle"
	set OuterSkinRLLP [lindex [IFileRead $path2Vehicle Vehicle.OuterSkin] 1]
#     Log "OuterSkinRLLP  $OuterSkinRLLP "
	set OuterSKinFURP [expr abs([lindex [IFileRead $path2Vehicle Vehicle.OuterSkin] 4])]
#     Log "OuterSKinFURP $OuterSKinFURP"
	set WRLPosY [lindex [IFileRead $path2Vehicle Wheel.rl.pos] 1];
	set TireWidthRL [IFileRead Tire2 NomWidth]
	
	set WFLPosY [lindex [IFileRead $path2Vehicle Wheel.fl.pos] 1];
	set TireWidthFL [IFileRead Tire0 NomWidth]
	
	set WidthRL [expr $WRLPosY+($TireWidthRL/2)]
	set WidthFL [expr $WFLPosY+($TireWidthFL/2)]
	
	if {$WidthFL > $WidthRL} {
		set Width $WidthFL
	} else {
		set Width $WidthRL
	}

	if {$TireWidthFL > $TireWidthRL} {
		set TireWidth $TireWidthFL
	} else {
		set TireWidth $TireWidthRL
	}
	
	set WFLRadius [IFileRead Tire0 KinRollRadius]
	set WFLDiagonal [expr (sqrt($WFLRadius*$WFLRadius+($TireWidth/2)*($TireWidth/2)))]
	
	NamedValue set Angle [expr asin($TireWidth/2/$WFLDiagonal)]
	
	if {$OuterSkinRLLP > 0 && ($OuterSkinRLLP >= $Width || $OuterSKinFURP >= $Width)} {
#         Log screen "# Vehicle Outer Skin defined - used for vehicle width"
		if {abs($OuterSkinRLLP) >= abs($OuterSKinFURP)} {
			set VehicleWidth [expr 2.0*$OuterSkinRLLP]
		} else {
			set VehicleWidth [expr 2.0*($OuterSKinFURP)]
		}
	} else {
#         Log screen "# Vehicle Outer Skin is not defined - wheel positions will be used for vehicle width"
#         set WRRPosY [lindex [IFileRead $path2Vehicle Wheel.rr.pos] 1];
#         Log "WRRPosY $WRRPosY"
#         offset between defined wheel position and outer vehicle position for width calculation
#         value is only estimated
		set WhlOffset 0.025;
#         set VehicleWidth [expr $WRLPosY - $WRRPosY + 2.0*$WhlOffset];
	set VehicleWidth [expr 2.0*($Width+$WhlOffset)]
#        Log screen "# estimated offset per wheel = $WhlOffset m"
	}
#     Log screen "# resulting vehicle width = [format %.3f $VehicleWidth] m"
	
	NamedValue set TireWidthFL $TireWidthFL
	NamedValue set WFLDiagonal $WFLDiagonal
	NamedValue set VehicleWidth $VehicleWidth

	return
}

### GetVehicleLength ###
#
# Determine length of vehicle under test & reference point x-coordinate for sRoad - coordinate (Fr1)
#
###
proc GetVehicleLength {args} {
	if {[KeyValue exists TestRun:Vehicle]} {
#       Log "Vehicle: [KeyValue get TestRun::Vehicle]"
		set path2Vehicle Data/Vehicle/[KeyValue get TestRun:Vehicle]

	} else {
		# use Vehicle linked with TestRun
		set path2TestRun Data/TestRun/TestWare/EuroNCAP_2020/Runs/AEB_Pedestrian/AEB_CPFA-50
#         Log screen "TestRun $path2TestRun"
		set vehicle [IFileRead $path2TestRun Vehicle]
#         Log screen "Vehicle: $vehicle"
		set path2Vehicle Data/Vehicle/$vehicle
	}
	
	set VehicleLength [lindex [IFileRead $path2Vehicle Vehicle.OuterSkin] 3]
	set WFLPosX [lindex [IFileRead $path2Vehicle Wheel.fl.pos] 0];
	set WFRPosX [lindex [IFileRead $path2Vehicle Wheel.fr.pos] 0];
	set WFLPosY [lindex [IFileRead $path2Vehicle Wheel.fl.pos] 1];
	set WFRPosY [lindex [IFileRead $path2Vehicle Wheel.fr.pos] 1];
	set WFLPosZ [lindex [IFileRead $path2Vehicle Wheel.fl.pos] 2];
	set WFRPosZ [lindex [IFileRead $path2Vehicle Wheel.fr.pos] 2];

	set VehicleRefPointsRoad [expr {($WFLPosX + $WFRPosX)/2}]
	set VehicleRefPointsRoadY [expr {($WFLPosY + $WFRPosY)/2}]
	set VehicleRefPointsRoadZ [expr {($WFLPosZ + $WFRPosZ)/2}]
	
#     Log screen "# Vehicle length = [format %.3f $VehicleLength] m"
#     Log screen "# Reference point for Vhcl.sRoad = [format %.3f $VehicleRefPointsRoad] m"

	NamedValue set VehicleLength $VehicleLength
	NamedValue set VehicleRefPointsRoad $VehicleRefPointsRoad
	NamedValue set VehicleRefPointsRoadY $VehicleRefPointsRoadY
	NamedValue set VehicleRefPointsRoadZ $VehicleRefPointsRoadZ
	
	return
}

# ### GetWRLPos ###
# #
# # Determine the x-Coordinate of the Rear Left Wheel centre
# #
# ###
# proc GetWRLPos {args} {
	# if {[KeyValue exists TestRun:Vehicle]} {
# #       Log "Vehicle: [KeyValue get TestRun::Vehicle]"
		# set path2Vehicle Data/Vehicle/[KeyValue get TestRun:Vehicle]

	# } else {
		# # use Vehicle linked with TestRun
		# set path2TestRun Data/TestRun/TestWare/EuroNCAP_2020/Runs/AEB_Pedestrian/AEB_CPFA-50
# #         Log screen "TestRun $path2TestRun"
		# set vehicle [IFileRead $path2TestRun Vehicle]
# #         Log screen "Vehicle: $vehicle"
		# set path2Vehicle Data/Vehicle/$vehicle
	# }
	
	# set WRLPosX [lindex [IFileRead $path2Vehicle Wheel.rl.pos] 0];
	# set WRRPosX [lindex [IFileRead $path2Vehicle Wheel.rr.pos] 0];
	
	# NamedValue set WRLPosX $WRLPosX
# }

### CompareResultDate ###
#
# Delete test results & all TS-variables if the date of test results
# differs more than one day from current date
#
###
proc CompareResultDate {args} {
	set difference_days 1
	set ::TS::Date "[clock format [clock seconds] -format %m/%d/%Y]"
	
	if {[info exists ::TS::ResultDate]} {
		set day1 [clock format [clock scan $::TS::ResultDate] -format %j]
		set day2 [clock format [clock scan $::TS::Date] -format %j]
	
		if {[expr {abs([scan $day2 %d] - [scan $day1 %d])}] > $difference_days} { 
			::TestMgr::ClearTS
			::TestMgr clearresults
		}
	}
}


### SetResultDate ###
#
# Store test result date in TS variables which is used in CompareResultDate for comparision
#
###
proc SetResultDate {args} {
	set ::TS::ResultDate "[clock format [clock seconds] -format %m/%d/%Y]"
}


### GetSystem ###
#
# Delete saved test scores (& optional: test results + all TS-variables),
# if the current system under test differs from previous setting
#
###
proc GetSystem {system args} {
    switch $system {
	ELKS {
		if {[info exists ::TS::System(ELKS)]} {
			if {[NamedValue get EquippedELKS] == 1} {
				set system ELKS
			}
		} else {
			if {[NamedValue get EquippedELKS] == 1} {
				set ::TS::System(ELKS) ELKS
			} else {
				set ::TS::System(ELKS) none
			}
		}
		set ::TS::ELKS [NamedValue get EquippedELKS]
		NamedValue set ELKS_On [set ::TS::ELKS]
	}
	default {}
    }
    
}

### SetSystem ###
#
# Store system information in TS variables which is used in GetSystem for comparision
#
###
proc SetSystem {system args} {
    switch $system {
	ELKS {
		if {![info exists ::TS::ELKS]} {
		    unset -nocomplain ::TS::System(ELKS)
		    return
		}
		if {$::TS::ELKS == 1} {
		    set ::TS::System(ELKS) ELKS
		} else {
		    set ::TS::System(ELKS) none
		}
	}
	default {}
    }
}



### SelectTSNamespace ###
#
# Selet used tcl namespace for test score/ result storage
#
###
proc SelectTSNamespace {args} {
global HIL
	
	if {$HIL(HPC.Active)} {
		set ns TS_tmp
	} else {
		set ns TS
	}
	
	return $ns
}

### End Namespace EuroNCAP ###
}