#! /bin/sh
# next line restarts using wish \
exec CM -wish "$0" "$@"
#
# Instruments.tcl
#
# Copyright (c) IPG Automotive GmbH, Karlsruhe, Germany (www.ipg-automotive.com)

set Pgm(Dir)  [file dirname [info script]]

set Pgm(Name)           "Instruments_ALKS"
set Pgm(Version)		"v1.0"

array set Config {
	MaxSpeed 	220
	MaxRPM		7000
	MaxStW		550
	nGears		6
	Mode		CM
	withPedals 	1
	Automatic   0
	Units		""
	FuelEff		""
}

# Subscription = list of variables to be subscriped
#   Tcl var             quantity        alternate quantities
set Subscription {
	{Qu(Distance) 		Vhcl.Distance}
	{Qu(vCar) 		IO.vCar			Vhcl.v }
	{Qu(nEngine) 		IO.nEngine		Vhcl.Engine.rotv}
	{Qu(Gas) 		VC.Gas			DM.Gas}
	{Qu(Clutch) 		VC.Clutch		DM.Clutch}
	{Qu(Brake) 		VC.Brake}
	{Qu(GearNo) 		DM.GearNo}
	{Qu(SelectorCtrl) 	DM.SelectorCtrl}
	{Qu(PT.GearNo) 		PT.GearBox.GearNo 	DM.GearNo}
	{Qu(StWhlAngle) 	Vhcl.Steer.Ang}
	{Qu(SpeedLimit) 	DM.SpeedLimit}
	{Qu(Consump_Abs)	PT.ECU.Consump_Abs  	PT.Engine.Consump_Abs}
	{Qu(Consump_Avg)	PT.ECU.Consump_Avg  	PT.Engine.Consump_Avg}
	{Qu(Consump_Act)	PT.ECU.Consump_Act  	PT.Engine.Consump_Act}
	{Qu(Kl15SW) 		Kl15SW }
	{Qu(SelManShift)	DM.SelectorManualShift}
	{Qu(SetGearNo)		PT.GearBox.Auto.SetGearNo}
	{Qu(EngineSwitch)	DM.SST	DM.EngineSwitch}
	{Qu(IndL)		VC.Lights.IndL}
	{Qu(IndR)		VC.Lights.IndR}
	{Qu(LaLKS)      	LatCtrl.LKAS.SwitchedOn}
	{Qu(LaLKSActive)   	LatCtrl.LKAS.IsActive}
	{Qu(LaLKSAutoPrio) 	LatCtrl.LKAS.UsePrioLines}
	{Qu(LaLKSTrq)   	LatCtrl.LKAS.AssistTrq}
	{Qu(LoAEB)   		LongCtrl.AEB.SwitchedOn}
	{Qu(LoAEBActive)	LongCtrl.AEB.IsActive}
	{Qu(LoACCActive)	AccelCtrl.ACC.IsActive}
	{Qu(LoACCTTC)		AccelCtrl.ACC.Time2Collision}
	{Qu(LoLDW)   		LatCtrl.LDW.SwitchedOn}
	{Qu(LoLDWActive)   	LatCtrl.LDW.IsActive}
	{Qu(LoCDCF)   		LatCtrl.LKAS.SwitchedOn}
	{Qu(LoCDCFActive)   LatCtrl.LKAS.IsActive}
	{Qu(ELKSvisual)   	LatCtrl.LDW.IsActive}
	{Qu(ELKSsound)      LatCtrl.LDW.IsActive}
}

set Qu(EngineSwitch) 0
set Qu(LaLKS) 		 0
set Qu(LaLDW)		 0

# max 10 Lights
set LEDs {
	0 "PWR" red
	1 "START\nSTOP" #0C0
	2 "LKS" green
	3 "AEB" green
	4 "ACC" green
}

set LEDs_MM {
	0 "PWR" red   
	2 "ABS" blue
	4 "(o)" red
	5 "BLS" red
}

array set lkascolors {
	0 grey70
	1 #aa3
	2 green
}
array set aebcolors {
	0 grey70
	1 green
	2 #c00
}
array set acccolors {
	0 grey70
	1 green
	2 #c00
}

## Handle args of command line ################################################

proc PrintUsage {} {
	global HIL Config
puts "
usage: Instruments \[options\]
-appclass c     restrict connections to appl class c    ($HIL(AcceptClass))
-appname n      .. to appl named n                      ($HIL(AcceptName))
-apphost h      .. to appl running on host h n          ($HIL(AcceptHost))
-appuser u      .. to appl owned by user u              ($HIL(AcceptUser))
-apppid pid     .. to appl process pid                  ($HIL(AcceptPid))
-connect        auto-connect after program start
-maxspeed v     set max speed                           ($Config(MaxSpeed))
-maxrpm v       set max RPM                             ($Config(MaxRPM))
-maxstw v       set max steering wheel angle            ($Config(MaxStW))
-ngears v       set number of gears                     ($Config(nGears))
-units s        configure system of measurement         ($Config(Units))
-fueleff e      configure fuel efficiency mode          ($Config(FuelEff))
-automatic      enable automatic gearbox visualization  ($Config(Automatic))
-MM             MotorCycleMaker mode
"
	exit 1
}

for {set i 0} {$i<$argc} {incr i} {
	set arg [lindex $argv $i]
	switch -glob $arg {
	-noPedals	{ set Config(withPedals) 0 }
	-apphost	{ set HIL(AcceptHost)  [lindex $argv [incr i]] }
	-appuser	{ set HIL(AcceptUser)  [lindex $argv [incr i]] }
	-appclass	{ set HIL(AcceptClass) [lindex $argv [incr i]] }
	-apppid		{ set HIL(AcceptPid)   [lindex $argv [incr i]] }
	-connect	{ after idle Connect}
	-maxspeed	{ set Config(MaxSpeed) [lindex $argv [incr i]] }
	-maxrpm		{ set Config(MaxRPM)   [lindex $argv [incr i]] }
	-maxst*		{ set Config(MaxStW)   [lindex $argv [incr i]] }
	-ngears		{ set Config(nGears)   [lindex $argv [incr i]] }
	-units		{ set Config(Units)    [lindex $argv [incr i]] }
	-fueleff   	{ set Config(FuelEff)  [lindex $argv [incr i]] }
	-automatic	{ set Config(Automatic) 1 }
	-lang		{  }
	-BM	-
	-MM       { set Config(Mode)     MM 
				set Config(MaxSpeed) 240
				set Config(MaxRPM)   12000
				set Config(MaxStW)   50
				set Config(nGears)   0
				set LEDs $LEDs_MM
			}
	-TM       { set Config(Mode)     TM
				set Config(MaxSpeed) 125
				set Config(MaxRPM)   3000
				set Config(MaxStW)   540
				set Config(nGears)   0
			}
	-*        { PrintUsage }
	}
}

set Config(RPMscale) [expr {$Config(MaxRPM) > 5500 ? 1000 : 100}]

proc MM {a b} {
	return [expr {$::Config(Mode)=="MM" ? $a : $b}]
}


## Load helper modules #######

if {![file exists $Pgm(Dir)/Tacho.tcl]} { append Pgm(Dir) "/../GUI/lib" } 
if {![file exists $Pgm(Dir)/Tacho.tcl]} { set Pgm(Dir) "$env(CM_HOME)/GUI/lib" }

load $LibDir/apoc.so

source $Pgm(Dir)/Tacho.tcl
source $Pgm(Dir)/HVBar.tcl
source $Pgm(Dir)/Gearbox.tcl


### Communication port for remote control (send/dde) ###

tk appname Instruments
if {$Pgm(Arch)=="win32"} { dde servername Instruments }


### APO communication ##################################################################

set ApoId ""

proc Subscribe {{sid ""}} {
	global ApoId HIL Qu Subscription

	foreach qu [dataprocess dictionary $ApoId] {
		set Dict([lindex $qu 0]) $qu
	}
	foreach sub $Subscription {
	set var [lindex $sub 0]
	for {set i 1} {$i < [llength $sub]} {incr i} {
		set qu [lindex $sub $i]
		if {[info exist Dict($qu)]} break
	}
		lappend Subs [list $qu $var]
	set $var 0
	}
	set ::vEngineSwitch [expr {[info exist Dict(DM.SST)] ? "DM.SST" : "DM.EngineSwitch"} ]
	set Rejected [dataprocess subscribe $ApoId 0 100 HandleDataVec $Subs]
	foreach i $Rejected {
	puts "Not available: #$i [lindex $Subs $i 0]"
	}
}

proc Connect {{Target ""}} {
	global ApoId Pgm HIL

		if {$Target != ""} {
		set HIL(AcceptPid) ""
		if {[scan $Target {%[^: ]:%d} HIL(AcceptHost) HIL(AcceptPid)] != 2} {
			if {[scan $Target {%u.%u} HIL(AcceptPid) dummy] == 1} { 
				set HIL(AcceptHost) localhost
			}
		}
	}

	apoc query 1000 $HIL(AcceptHost)
	while {![apoc querydone]} {after 100; update}

	set SelIndex      -1
	set Sel(AppStart) 0
	set Sel(AppHost)  ""
	set i 0
	foreach s [apoc servers] {
		lassign $s Info(AppClass) Info(AppName) Info(AppInfo) dummy Info(AppDescr) \
			Info(AppStart) Info(AppUser) Info(AppHost) Info(AppPid)
		if {($HIL(AcceptUser) == ""  || $HIL(AcceptUser)==$Info(AppUser))
		&& ($HIL(AcceptClass) == "" || $HIL(AcceptClass)==$Info(AppClass))
		&& ($HIL(AcceptPid) == ""   || $HIL(AcceptPid)==$Info(AppPid))
		&& $Info(AppStart) > $Sel(AppStart)} {
			set SelIndex $i
			array set Sel [array get Info]
		}
		incr i
	}
	if {$SelIndex < 0} {return 0}
	Disconnect

	if {[catch {dataprocess new $SelIndex 33} ApoId] != 0} {
		wm title . "$Pgm(Name) - could not connect"
		set ApoId ""
		return
	}
	dataprocess tracemsg $ApoId 5 RecvMsg
	dataprocess dictchanged $ApoId Subscribe
	wm title . "$Pgm(Name) - '$Sel(AppHost)' online"

	Subscribe
	set ::Qu(SpeedLimit) -1
	SendRequest
}


proc Disconnect {} {
	catch {dataprocess delete $::ApoId}
	set ::ApoId ""
	wm title . "$::Pgm(Name) - disconnected"
}


proc SendMsg {Channel Msg} {
	catch {dataprocess sendmsg $::ApoId $Channel $Msg}
}


proc RecvMsg {Process ch Msg} {
	binary scan $Msg S Id
	if {$Id == 0} {
		set n [binary scan $Msg x2ccSSRR GearBoxKind BrakeKind nFGears nBGears totvEngineMax velVhclMax]
	#Gearbox::Configure .gear.no -n $nFGears -auto $GearBoxKind
	#Tacho::Configure .rpm -max $totvEngineMax
	#Tacho::Configure .spd -max $velVhclMax
	#HVBar::Configure .mid.clutch -auto $GearBoxKind
	}
}


proc SendDVAwr {QuName Value {Where 0} {Mode 0} {nActive 1}} {
	set msg [binary f a8I "DVAwr   " 123]
	# Type(f/i) Where(in/out/DM) Mode(val/offset/factor) nActive QName
	append msg [binary f accIa*xR f $Where $Mode $nActive $QuName $Value]
	SendMsg 2 $msg
}

proc ReleaseDVAwr {{QuName "*"}} {
	set msg  [binary f a8I "DVAwr   " 123]
	append msg [binary f accIa*xI f 0 0 0 $QuName 0]
	SendMsg 2 $msg
}

proc SendRequest {} {
	set msg [binary f S 1000]
	SendMsg 5 $msg
}

### APO communication ##################################################################


proc HandleDataVec {sid msgid} {

	global Config Qu LED lkascolors aebcolors acccolors

	Tacho::SetNeedle .rpm 0 [expr {9.549*$Qu(nEngine)/$Config(RPMscale)}]
	Tacho::SetNeedle .spd 0 [CalcDist [expr {$Qu(vCar)*3.6}]]

	set vLimit $Qu(SpeedLimit)
	
	Tacho::SetNeedle .spd 1 [CalcDist [expr {$vLimit}]]

	.spd itemconfig km -text [format "%.3f" [CalcDist [expr {$Qu(Distance)*0.001}]]]
	.spd itemconfig indL -fill [expr {$Qu(IndL) > 1 ? {#1e1} : {grey20} }]
	.spd itemconfig indR -fill [expr {$Qu(IndR) > 1 ? {#1e1} : {grey20} }]
	.rpm itemconfig gear -text [Gearbox::GearName $Qu(PT.GearNo)]

	# consumption display
	.rpm itemconfig cact -text [CalcFuelEff $Qu(Consump_Act)]
	.rpm itemconfig cavg -text [CalcFuelEff $Qu(Consump_Avg)]
	if {$Qu(vCar)<1}      { .rpm itemconfig cact -text "" }
	if {$Qu(Distance)<10} { .rpm itemconfig cavg -text "" }

	# AEB status
	set w .sysinfo.long
	if {$Qu(LoAEB) == 1} {
		.buttonbar.buttons_alks.aeb.l configure -bg #0b0
		.buttonbar.buttons_alks.aeb.l configure -fg #fff
		.buttonbar.buttons_alks.aeb.l configure -text "On"
		if {$Qu(LoAEBActive) == 1} {
			$w itemconfig aeb_status -text "ACTIVE"
			$w itemconfig aeb_status -fill orange
		} else {
			$w itemconfig aeb_status -text "On"
			$w itemconfig aeb_status -fill #6f6
		}
	} else {
		.buttonbar.buttons_alks.aeb.l configure -bg #ccc
		.buttonbar.buttons_alks.aeb.l configure -fg #333
		.buttonbar.buttons_alks.aeb.l configure -text "Off"
		$w itemconfig aeb_status -text "Off"
		$w itemconfig aeb_status -fill #f00
	}

	# ACC status
	if {$Qu(LoACCActive) == 1} {
		$w itemconfig acc_status -text "ACTIVE"
		$w itemconfig acc_status -fill orange
	} else {
		$w itemconfig acc_status -text "On"
		$w itemconfig acc_status -fill #6f6
	}

	# LKAS status
	set w .sysinfo.lat
	if {$Qu(LaLKS) == 1} {
		.buttonbar.buttons_alks.lat.l configure -bg #0b0
		.buttonbar.buttons_alks.lat.l configure -fg #fff
		.buttonbar.buttons_alks.lat.l configure -text "On"
		$w itemconfig lkas_trq -text [format "%3.1f" $Qu(LaLKSTrq)]
		if {$Qu(LaLKSAutoPrio) == 1} {
			$w itemconfig lkas_autoprio -text "On"
			$w itemconfig lkas_autoprio -fill #6f6
			.buttonbar.buttons_alks.prioritize.l configure -bg #0b0
			.buttonbar.buttons_alks.prioritize.l configure -fg #fff
			.buttonbar.buttons_alks.prioritize.l configure -text "On"
		} else {
			$w itemconfig lkas_autoprio -text "Off"
			$w itemconfig lkas_autoprio -fill #fc0
			.buttonbar.buttons_alks.prioritize.l configure -bg #ccc
			.buttonbar.buttons_alks.prioritize.l configure -fg #333
			.buttonbar.buttons_alks.prioritize.l configure -text "Off"
		}
		if {$Qu(LaLKSActive) == 1 } {
			$w itemconfig lkas_status -text "On"
			$w itemconfig lkas_status -fill #6f6
			.spd itemconfigure lkas_icon -state normal
		} else {
			$w itemconfig lkas_status -text "Passive"
			$w itemconfig lkas_status -fill #fc0
			.spd itemconfigure lkas_icon -state disabled
		}
	} else {
		.buttonbar.buttons_alks.lat.l configure -bg #ccc
		.buttonbar.buttons_alks.lat.l configure -fg #333
		.buttonbar.buttons_alks.lat.l configure -text "Off"
		.buttonbar.buttons_alks.prioritize.l configure -bg #ccc
		.buttonbar.buttons_alks.prioritize.l configure -fg #333
		.buttonbar.buttons_alks.prioritize.l configure -text "Off"
		$w itemconfig lkas_status -text "Off"
		$w itemconfig lkas_status -fill #ccc
		$w itemconfig lkas_trq -text ""
		$w itemconfig lkas_autoprio -text ""
		.spd itemconfigure lkas_icon -state hidden
	}
	
	# LDW status
	set w .sysinfo.long
	if {$Qu(LoLDW) == 1} {
		.buttonbar.buttons_alks.elks.l configure -bg #0b0
		.buttonbar.buttons_alks.elks.l configure -fg #fff
		.buttonbar.buttons_alks.elks.l configure -text "On"
		if {$Qu(LoLDWActive) == 1} {
			$w itemconfig ldw_status -text "ACTIVE"
			$w itemconfig ldw_status -fill orange
		} else {
			$w itemconfig ldw_status -text "On"
			$w itemconfig ldw_status -fill #6f6
		}
	} else {
		.buttonbar.buttons_alks.elks.l configure -bg #ccc
		.buttonbar.buttons_alks.elks.l configure -fg #333
		.buttonbar.buttons_alks.elks.l configure -text "Off"
		$w itemconfig ldw_status -text "Off"
		$w itemconfig ldw_status -fill #f00
	}
	
	# LDW visual signal
	if {$Qu(ELKSvisual) == 1} {
		.rpm itemconfigure ldw_icon -state normal
	} else {
		.rpm itemconfigure ldw_icon -state hidden
	}
	
	# LDW visual signal
	if {$Qu(ELKSsound) == 1} {
		.rpm itemconfigure sound_warn -state normal
	} else {
		.rpm itemconfigure sound_warn -state hidden
	}
	
	
	# CDCF status
	set w .sysinfo.long
	if {$Qu(LoCDCF) == 1} {
		if {$Qu(LoCDCFActive) == 1} {
			$w itemconfig cdcf_status -text "ACTIVE"
			$w itemconfig cdcf_status -fill orange
		} else {
			$w itemconfig cdcf_status -text "On"
			$w itemconfig cdcf_status -fill #6f6
		}
	} else {
		$w itemconfig cdcf_status -text "Off"
		$w itemconfig cdcf_status -fill #f00
	}
	
}

################################################################################

proc CreateDialog {} {
	global simData Config Pref Pgm env
	upvar #0 simData v
	
	# relative path
	set Dir [file dirname [info script]]
	
	if {![file exists "$Dir/InstrumentIcons/lkas_passive.gif"]} { 
		# absolute path to instalation directory
		set Dir "$env(CM_HOME)/bin/" 
	}
	
	image create photo lkas_passive 	-file "$Dir/InstrumentIcons/lkas_passive.gif"
	image create photo lkas_active  	-file "$Dir/InstrumentIcons/lkas_active.gif"
	image create photo acc_active		-file "$Dir/InstrumentIcons/acc_active.png"
	image create photo acc_passive		-file "$Dir/InstrumentIcons/acc_passive.png"
    # image create photo ldw_active  	    -file "$Dir/InstrumentIcons/ldw_active.gif"
	image create photo ldw_passive  	-file "$Dir/InstrumentIcons/ldw_passive.gif"
	image create photo ldw_warn      	-file "$Dir/InstrumentIcons/ldw_warn.gif"
	image create photo sound_icon      	-file "$Dir/InstrumentIcons/sound_icon.png"

	if {$Config(FuelEff) eq ""} { set Config(FuelEff) $Pref(FuelEff) }
	if {$Config(Units) eq ""} { set Config(Units) $Pref(Units) }

	TopLevel . "$Pgm(Version)"
	wm iconname . $Pgm(Name)
	wm protocol  . WM_DELETE_WINDOW ProgramExit
	wm resizable . 0 0

	#### Frame .top ####
	frame .top     -relief flat -bd 1 -bg black

	### Tacho ###
	Tacho::Create .spd -width 250 -height 200 -scaledist 0.75 \
		-max 270 \
		-major [expr {$Config(MaxSpeed)<130 ? 10 : 20}] -nminor 4 \
		-needle 0 {0.1  0.9 4 #cc3333} \
		-needle 1 {0.94 0.99 3 red} \
		-needle 2 {0.88 0.92 5 #0C0}
	.spd configure -bg black
	.spd create text 125 100 -text "km/h" -anchor center -fill white \
		-font {Sans -11} -tags {bg speed}
	.spd create rect 92 169 159 186 -fill grey20 -outline grey40 -tags bg
	.spd create text 156 178 -text "0.000" -anchor e -fill grey90 \
		-font {Sans -14 bold} -tags km
	.spd create rect 105 70 112 80 -fill grey20 -width 0 -tags indL
	.spd create polygon 95 75 105 65 105 85 -fill grey20 -width 0 -tag indL
	.spd create rect 138 70 145 80 -fill grey20 -width 0 -tags indR
	.spd create polygon 155 75 145 65 145 85 -fill grey20 -width 0 -tag indR
	.spd lower bg needles
	.spd create image 167 150 -image lkas_active -disabledimage lkas_passive -tag lkas_icon
	.spd create image 167 120 -image acc_active -disabledimage acc_passive -tag acc_icon

	### Tacho ###

	### Motor RPM ###
	Tacho::Create .rpm -width 250 -height 200 \
		-needle 0 {0.1  0.9 4 #cc3333} \
		-max   [expr {$Config(MaxRPM)/$Config(RPMscale)}] \
		-major [expr {$Config(MaxRPM)<4000 ? 5 : 1000/$Config(RPMscale)}] \
		-nminor 5
	.rpm configure -bg black
	.rpm create text 125 93 -text "[l "rpm" "U/min"]\nx $Config(RPMscale)" \
			-anchor center -ju c -fill white -font {Sans -11} -tags {bg rpm}
	.rpm create rect  87 157 116 188 -fill grey20 -outline grey40 -tags bg
	.rpm create rect 116 157 168 188 -fill grey20 -outline grey40 -tags bg
	.rpm create text 102 174 -anchor c -fill grey90 -font {Sans -24 bold} -tags gear
	.rpm create text 166 166 -anchor e -fill grey90 -font {Sans -13 bold} -tags cact
	.rpm create text 120 179 -anchor w -fill grey80 -font {Sans -13 bold} -text ""
	.rpm create text 166 180 -anchor e -fill grey90 -font {Sans -13 bold} -tags cavg
	.rpm create text 166 155 -text "l / 100 km" -anchor se -fill grey90 -font {Sans -9} -tag fuel
	.rpm lower bg needles
	.rpm create image 180 120 -image ldw_warn -disabledimage ldw_passive -tag ldw_icon
	.rpm create image 70 120 -image sound_icon -disabledimage sound_icon -tag sound_warn
	### Motor RPM ###
	
	### System info display ###
	set w .sysinfo
	frame $w -height 82 -width 180 -relief sunken -bd 3 -bg #000
	
	## Sysinfo Longitudinal ##
	set s long
	canvas $w.$s -height 35 -width 171 -relief flat -bg #000
	$w.$s create rectangle 0 0 170 34 -outline #ccc
	$w.$s create text 5 3 -text "ACC" -anchor nw -fill #fff -font {Sans -11 bold}
	$w.$s create text 45 2 -text "--" -anchor nw -fill #6f6 -font {Sans -11 bold} -tags acc_status
	$w.$s create text 5 18 -text "AEB" -anchor nw -fill #fff -font {Sans -11 bold}
	$w.$s create text 45 18 -text "--" -anchor nw -fill #6f6 -font {Sans -11 bold} -tags aeb_status
	$w.$s create text 90 3 -text "LDW" -anchor nw -fill #fff -font {Sans -11 bold}
	$w.$s create text 127 2 -text "--" -anchor nw -fill #6f6 -font {Sans -11 bold} -tags ldw_status
	$w.$s create text 90 18 -text "CDCF" -anchor nw -fill #fff -font {Sans -11 bold}
	$w.$s create text 127 18 -text "--" -anchor nw -fill #6f6 -font {Sans -11 bold} -tags cdcf_status
	place $w.$s -x 2 -y 1
	## Sysinfo Longitudinal ##

	## Sysinfo Lateral ##
	set s lat
	canvas $w.$s -height 35 -width 171 -relief flat -bg #000
	$w.$s create rectangle 0 0 170 34 -outline #ccc
	$w.$s create text 5   3 -text "LatCtrl" -anchor nw -fill #fff  -font {Sans -11 bold}
	$w.$s create text 45   3 -text "--" -anchor nw -fill #6f6  -font {Sans -11 bold} -tags lkas_status
	$w.$s create text 88   3 -text "AddTrq:" -anchor nw -fill #fff  -font {Sans -11}
	$w.$s create text 147  3 -text "---" -anchor ne -fill #fff  -font {Sans -11 bold} -tags lkas_trq
	$w.$s create text 150  3 -text "Nm" -anchor nw -fill #fff  -font {Sans -11}
	$w.$s create text 88  18 -text "Auto Prio:" -anchor nw -fill #fff  -font {Sans -11}
	$w.$s create text 150  18 -text "--" -anchor nw -fill #6f6  -font {Sans -11 bold} -tags lkas_autoprio
	place $w.$s -x 2 -y 40
	## Sysinfo Lateral ##
	
	### System info display ###
	
	### Speedometer - System info dislay  - Motor RPM ### 
	grid .spd 	-in .top -row 0 -column 0 -sticky wes -padx 2
	grid .sysinfo 	-in .top -row 0 -column 1 -sticky wes -padx 6 -pady 20
	grid .rpm 	-in .top -row 0 -column 2 -sticky wes -padx 2
	grid .top -columnspan 3
	
	### Button bar ###
	set w .buttonbar
	frame $w -height 100 -width 300 -relief flat -bd 0 -bg #000

	## ALKS ON/Off button ##
	set cv buttons_alks
	canvas $w.$cv -width 230 -height 83 -relief flat -bd 0 -bg #000
	$w.$cv create rectangle 1 10 226 80 -outline #999
	$w.$cv create rect 10 0 70 20 -fill #000
	$w.$cv create text 15 4 -text "ALKS" -anchor nw -fill #fff  -font {Sans -12 bold}
	# AEB Button #
	$w.$cv create text 15 30 -text "AEB" -anchor nw -fill #fff  -font {Sans -12}
	frame $w.$cv.aeb -relief ridge -bd 1
	label $w.$cv.aeb.l -fg #333 -text "Off" -font {Sans -13 bold}
	pack  $w.$cv.aeb.l -fill both -expand 1
	bind  $w.$cv.aeb.l <ButtonPress>   {SendDVAwr LongCtrl.AEB.SwitchedOn [expr {!$Qu(LoAEB)}]
		%W configure -relief sunken }
	bind  $w.$cv.aeb.l <ButtonRelease> {%W configure -relief flat}
	$w.$cv create window 60 27 -anchor nw -width 35 -height 20 -window $w.$cv.aeb
	# Lateral Button #
	$w.$cv create text 15 55 -text "LatCtrl" -anchor nw -fill #fff  -font {Sans -12}
	frame $w.$cv.lat -relief ridge -bd 1
	label $w.$cv.lat.l -fg #333 -text "Off" -font {Sans -13 bold}
	pack  $w.$cv.lat.l -fill both -expand 1
	bind  $w.$cv.lat.l <ButtonPress>   {SendDVAwr LatCtrl.LKAS.SwitchedOn [expr {!$Qu(LaLKS)}]
		%W configure -relief sunken }
	bind  $w.$cv.lat.l <ButtonRelease> {%W configure -relief flat}
	$w.$cv create window 60 52 -anchor nw -width 35 -height 20 -window $w.$cv.lat
	place $w.$cv -x 118 -y 5
	# Auto Prioritize Lane Button #
	$w.$cv create text 114 21 -text "Auto Prio.\nLane" -anchor nw -fill #fff  -font {Sans -12}
	frame $w.$cv.prioritize -relief ridge -bd 1
	label $w.$cv.prioritize.l -fg #333 -text "Off" -font {Sans -13 bold}
	pack  $w.$cv.prioritize.l -fill both -expand 1
	bind  $w.$cv.prioritize.l <ButtonPress>   { SendDVAwr LatCtrl.LKAS.UsePrioLines [expr {!$Qu(LaLKSAutoPrio)}]
		%W configure -relief sunken }
	bind  $w.$cv.prioritize.l <ButtonRelease> {%W configure -relief flat}
	$w.$cv create window 167 27 -anchor nw -width 35 -height 20 -window $w.$cv.prioritize
	place $w.$cv -x 233 -y 5
	#ELKS Lane Button #
	$w.$cv create text 114 55 -text "ELKS" -anchor nw -fill #fff  -font {Sans -12}
	frame $w.$cv.elks -relief ridge -bd 1
	label $w.$cv.elks.l -fg #333 -text "Off" -font {Sans -13 bold}
	pack  $w.$cv.elks.l -fill both -expand 1
	bind  $w.$cv.elks.l <ButtonPress>   { SendDVAwr LatCtrl.LDW.SwitchedOn [expr {!$Qu(LoLDW)}]
		%W configure -relief sunken }
	bind  $w.$cv.elks.l <ButtonRelease> {%W configure -relief flat}
	$w.$cv create window 167 52 -anchor nw -width 35 -height 20 -window $w.$cv.elks
	place $w.$cv -x 233 -y 5

	## ALKS ON/OFF button ##
		
	grid .buttonbar -in .top -row 1 -columnspan 4 -sticky news
	### Button bar ###
	#### Frame .top ####

	### Initialize Instruments ###
	Tacho::SetNeedle .spd 0 0
	Tacho::SetNeedle .rpm 0 0
	
	### Context menu ###
	set m [menu .cmenu -tearoff 0]
	$m add radiobutton -lab "Metric units" -variable Config(Units) \
	-value "metric" -command Configure
	$m add radiobutton -lab "Imperial units" -variable Config(Units) \
	-value "imperial" -command Configure
	$m add radiobutton -lab "US-American units" -variable Config(Units) \
	-value "american" -command Configure
	$m add separator
	$m add cascade -lab "Fuel Efficiency" -menu $m.fuel
	menu $m.fuel -tearoff 0
	$m.fuel add radiobutton -lab "litres / 100 km" -variable Config(FuelEff) \
	-value "consumption" -command Configure
	$m.fuel add radiobutton -lab "km / litre" -variable Config(FuelEff) \
	-value "economy" -command Configure
	$m add separator
	$m add com -lab "Connect"    -u 0 -comm Connect
	$m add com -lab "Disconnect" -u 0 -comm Disconnect
	$m add com -lab "Exit"       -u 2 -comm ProgramExit
	bind all <Button-3> "tk_popup .cmenu %X %Y"
	Configure
}

proc Configure {} {
	global Config
	Tacho::Configure .spd -max $Config(MaxSpeed)
	if {$Config(Units) eq "metric"} {
	set stext "km/h"
	if {$Config(FuelEff) eq "consumption"} {
		set ltext "l / 100 km"
	} else {
		set ltext " km / l   "
	}
	.cmenu.fuel entryconfigure 0 -lab "litres / 100 km"
	.cmenu.fuel entryconfigure 1 -lab "km / litre"
	} else {
	set stext "MPH"
	if {$Config(FuelEff) eq "consumption"} {
		set ltext "   GPM    "
	} else {
		set ltext "   MPG    "
	}
	.cmenu.fuel entryconfigure 0 -lab "gallons / 100 miles"
	.cmenu.fuel entryconfigure 1 -lab "miles / gallon"
	}
	.rpm itemconfigure fuel -text $ltext
	.spd itemconfigure speed -text $stext
}


proc CalcFuelEff {val} {
	global Config
	if {$Config(FuelEff) == "economy" && $val == 0} { return "" }
	switch [list $Config(Units) $Config(FuelEff)] {
	{metric consumption}	{ }
	{metric economy}	{ set val [expr {100.0/$val} ] } 
	{imperial consumption}	{ set val [expr {$val/2.8248} ] }
	{imperial economy}	{ set val [expr {282.48/$val} ] }
	{american consumption}	{ set val [expr {$val/2.3521} ] }
	{american economy}	{ set val [expr {235.21/$val} ] }
	}
	return [format "%.2f" [expr {min(99.99, $val)} ]]
}


proc CalcDist {val} {
	return [expr {$::Config(Units) eq "metric" ? $val : 0.621*$val} ]
}


### Preferences ###################################

array set Pref {
	Window.Main	""
	FuelEff		"consumption"
	Units		"metric"
}
catch {source ~/.instruments.rc}


proc PrefSave {} {
	global Pref Config
	scan [wm geo .] "%*dx%*d%s" Pref(Window.Main)
	set Pref(FuelEff) $Config(FuelEff)
	set Pref(Units) $Config(Units)
	catch {
		set fd [open "~/.instruments.rc" w]
		foreach n [array names Pref] { puts $fd "set Pref($n) $Pref($n)" }
	close $fd
	}
}
### Preferences ###################################

### MAIN ##########################################

proc ProgramExit {} {
	PrefSave
	exit
}

CreateDialog
bind all <Key-F2> 	{SendMsg 2 "Kl15\0"}
bind all <Control-Key-r> Connect
bind all <Control-Key-q> ProgramExit
bind all <Alt-Key-x>     ProgramExit

apoc pollinterval 100

### MAIN ##########################################